@@ -1,4 +1,4 @@
-/*  $Id: GameEngine.cpp,v 1.2 2004/04/21 02:19:14 sarrazip Exp $
+/*  $Id: GameEngine.cpp,v 1.5 2008/11/16 05:22:30 sarrazip Exp $
     GameEngine.cpp - Abstract class representing an X11 Game Engine.
 
     flatzebra - Generic 2D Game Engine library
@@ -43,12 +43,19 @@ GameEngine::GameEngine(Couple screenSizeInPixels,
 
     SDL_WM_SetCaption(wmCaption.c_str(), wmCaption.c_str());
 
+    Uint32 flags = SDL_HWSURFACE | SDL_ANYFORMAT;
+    if (fullScreen)
+	flags |= SDL_FULLSCREEN;
+
     theDepth = SDL_VideoModeOK(
 			screenSizeInPixels.x, screenSizeInPixels.y,
-			32, SDL_HWSURFACE);
-    Uint32 flags = (fullScreen
-			? SDL_FULLSCREEN
-			: SDL_HWSURFACE);
+			32, flags);
+    if (theDepth <= 0)
+    {
+	SDL_Quit();
+	throw string("video mode not available");
+    }
+
     theSDLScreen = SDL_SetVideoMode(
     			screenSizeInPixels.x, screenSizeInPixels.y,
 			theDepth, flags);
@@ -89,6 +96,7 @@ GameEngine::GameEngine(Couple screenSizeInPixels,
 
 GameEngine::~GameEngine()
 {
+    SDL_FreeSurface(fixedWidthFontPixmap);
     SDL_Quit();
 }
 
@@ -125,7 +133,7 @@ void GameEngine::run(int millisecondsPerFrame)
 
 
 void
-GameEngine::loadPixmap(char **xpmData, PixmapArray &pa, size_t index) const
+GameEngine::loadPixmap(const char **xpmData, PixmapArray &pa, size_t index) const
 						throw(PixmapLoadError)
 {
     // Masks are not be relevant with SDL.
@@ -139,7 +147,7 @@ GameEngine::loadPixmap(char **xpmData, PixmapArray &pa, size_t index) const
 
 
 void
-GameEngine::loadPixmap(char **xpmData,
+GameEngine::loadPixmap(const char **xpmData,
 		SDL_Surface *&pixmap,
 		Couple &pixmapSize) const throw(PixmapLoadError)
 {
@@ -148,7 +156,7 @@ GameEngine::loadPixmap(char **xpmData,
     if (xpmData == NULL || xpmData[0] == NULL)
 	throw PixmapLoadError(PixmapLoadError::INVALID_ARGS, NULL);
 
-    pixmap = IMG_ReadXPMFromArray(xpmData);
+    pixmap = IMG_ReadXPMFromArray(const_cast<char **>(xpmData));
     if (pixmap == NULL)
 	throw PixmapLoadError(PixmapLoadError::UNKNOWN, NULL);
 
@@ -170,7 +178,7 @@ GameEngine::writeString(const char *s, Couple pos, SDL_Surface *surface)
     for (size_t i = 0; s[i] != '\0'; i++, dest.x += fontDim.x)
     {
 	unsigned char c = (unsigned char) s[i];
-	if (c < 32 || c >= 127 && c <= 160)  // if ctrl char or undef char
+	if (c < 32 || (c >= 127 && c <= 160))  // if ctrl char or undef char
 	    c = 32;  // replace by space
 
 	/*  Compute the subrectangle of fixedWidthFontPixmap that
